import click
import google.generativeai as palm
import llm
from pprint import pprint


@llm.hookimpl
def register_models(register):
    register(Palm("chat-bison-001"), aliases=("palm", "palm2"))


@llm.hookimpl
def register_commands(cli):
    @cli.group(name="palm")
    def palm_():
        "Commands for working directly with PaLM"

    @palm_.command()
    @click.option("--key", help="PaLM API key")
    def models(key):
        "List models available in the PaLM API"
        api_key = llm.get_key(key, "palm", "PALM_API_KEY")
        palm.configure(api_key=api_key)
        models = palm.list_models()
        pprint(list(models))


class Palm(llm.Model):
    needs_key = "palm"
    key_env_var = "PALM_API_KEY"

    def __init__(self, model_id):
        self.model_id = model_id

    def build_prompt_messages(self, prompt, conversation):
        if not conversation:
            return prompt
        messages = []
        for response in conversation.responses:
            messages.append(response.prompt.prompt)
            messages.append(response.text())
        messages.append(prompt)
        return messages

    def execute(self, prompt, stream, response, conversation):
        palm.configure(api_key=self.get_key())

        kwargs = {"messages": self.build_prompt_messages(prompt.prompt, conversation)}
        if prompt.system:
            kwargs["context"] = prompt.system

        palm_response = palm.chat(**kwargs)
        last = palm_response.last
        yield last or ""
        response._prompt_json = kwargs

    def __str__(self):
        return "PaLM 2: {}".format(self.model_id)



import ast
from click.testing import CliRunner
from unittest.mock import patch, Mock
from llm.cli import cli
from llm_palm import Palm
from llm import Prompt, get_model, Response
import os
import pytest
from typing import List, Tuple


@patch("llm_palm.palm")
def test_palm_response(mock_palm):
    mock_response = Mock()
    mock_response.last = "hello"
    mock_palm.chat.return_value = mock_response
    prompt = Prompt("hello", "")
    model = Palm("palm2")
    model.key = "key"
    items = list(model.response(prompt))

    mock_palm.chat.assert_called_with(messages="hello")

    assert items == ["hello"]


@patch.dict(os.environ, {"PALM_API_KEY": "X"})
@patch("llm_palm.palm")
def test_palm_models(mock_palm):
    fake_models = [
        {"name": "'models/chat-bison-001"},
        {"name": "models/text-bison-001"},
    ]
    # Mock palm.list_models()
    mock_palm.list_models.return_value = fake_models
    runner = CliRunner()
    result = runner.invoke(cli, ["palm", "models"])
    assert result.exit_code == 0, result.output
    assert ast.literal_eval(result.output) == fake_models


@patch.dict(os.environ, {"PALM_API_KEY": "X"})
@patch("llm_palm.palm")
def test_palm_prompt(mock_palm):
    mock_response = Mock()
    mock_response.last = "🐶🐶"
    mock_palm.chat.return_value = mock_response
    runner = CliRunner()
    result = runner.invoke(cli, ["two dog emoji", "-m", "palm"])
    assert result.exit_code == 0, result.output
    assert result.output == "🐶🐶\n"


@pytest.mark.parametrize(
    "prompt,conversation_messages,expected",
    (
        ("hello", [], "hello"),
        ("hello 2", [("user 1", "response 1")], ["user 1", "response 1", "hello 2"]),
        (
            "hello 3",
            [("user 1", "response 1"), ("user 2", "response 2")],
            ["user 1", "response 1", "user 2", "response 2", "hello 3"],
        ),
    ),
)
def test_build_prompt_messages(
    prompt: str, conversation_messages: List[Tuple[str, str]], expected: List[str]
):
    model = get_model("palm2")
    conversation = None
    if conversation_messages:
        conversation = model.conversation()
        for prev_prompt, prev_response in conversation_messages:
            conversation.responses.append(
                Response.fake(
                    prompt=prev_prompt,
                    model=model,
                    system=None,
                    response=prev_response,
                )
            )

    messages = model.build_prompt_messages(prompt, conversation)
    assert messages == expected
